
def use_ipdb():
    # TODO: make `from kpa import use_ipdb` set `sys.excepthook` without `use_ipdb()`?
    import sys
    def excepthook(*args, **kwargs):
        from IPython.core import ultratb
        sys.excepthook = ultratb.FormattedTB(mode='Verbose', color_scheme='Linux', call_pdb=1)
        return sys.excepthook(*args, **kwargs)
    sys.excepthook = excepthook


def termcolor(text, fg=None, bg=None):
    from subprocess import check_output
    def get_code(args):
        if not hasattr(termcol,'_cache'): termcol._cache = {}
        if termcol._cache.get(args, None) is None: termcol._cache[args] = check_output(f'tput {args}'.split()).decode()
        return termcol._cache[args]
    return (get_code(f'setaf {fg}') if fg else '') + (get_code(f'setab {bg}') if bg else '') + text + get_code('sgr0')


class TerminalLineWrapper:
    # TODO: make these classmethods
    '.write_text(text) wraps long lines using an eight-space indent and returns the result'
    def __init__(self):
        import shutil
        self.cols = shutil.get_terminal_size().columns
    def write_text(self, text):
        for line in text.split('\n'):
            self._write_line(line)
    def _write_line(self, text):
        ret = ''
        x = self._how_many_chars_fit_in_width(text, self.cols)
        ret += (text[:x])
        while x < len(text):
            text = text[x:]
            x = self._how_many_chars_fit_in_width(text, self.cols - 8)
            ret += '\n' + ' '*8 + text[:x]
        return ret
    def _how_many_chars_fit_in_width(self, s, width):
        for i, c in enumerate(s):
            width -= self._width_of_char(c)
            if width < 0: return i
        return len(s)
    def _width_of_char(self, c):
        import unicodedata
        ## A: ambiguous (1 in iterm)
        ## F: full width (2)
        ## H: halfwidth (1)
        ## N : not asian (1)
        ## Na: narrow (1)
        ## W: wide (2)
        u_eaw = unicodedata.east_asian_width(c)
        if u_eaw in ('H','N','Na','A'): return 1
        elif u_eaw in ('F','W'): return 2
        else: raise Exception(ord(c))




#!/usr/bin/env python3
# to install: `pip install -e .`
# to upgrade: `pip3 install --upgrade --upgrade-strategy eager --no-cache-dir kpa`
# to publish: `python3 setup.py publish`
# to test: `python3 setup.py test`


from setuptools import setup
import imp
import sys

if sys.argv[-1] == 'publish':
    import pathlib, subprocess
    # TODO: auto-increment version
    if not pathlib.Path('~/.pypirc').expanduser().exists(): print('warning: you need ~/.pypirc')
    if pathlib.Path('dist').exists() and list(pathlib.Path('dist').iterdir()): print('warning: dist/* may cause problems')
    subprocess.check_output('python3 setup.py sdist bdist_wheel'.split())
    subprocess.check_output('twine upload dist/*'.split())


version = imp.load_source('kpa.version', 'kpa/version.py').version

setup(
    name='kpa',
    version=version,
    description="<forthcoming>",
    author="Peter VandeHaar",
    author_email="pjvandehaar@gmail.com",
    url="https://github.com/pjvandehaar/kpa",
    classifiers=[
        'Programming Language :: Python :: 3 :: Only',
        'Operating System :: Unix',
    ],

    packages=['kpa'],
    entry_points={'console_scripts': [
        'kpa=kpa.command_line:main',
    ]},
    include_package_data=True,
    zip_safe=False,
    python_requires=">=3.6",
    setup_requires=[
        'pytest-runner~=4.0',
    ],
    install_requires=[],
    tests_require=[
        'pytest~=3.4',
    ],
)
